@@ -1,7 +1,7 @@
 #
 # Monitorix - A lightweight system monitoring tool.
 #
-# Copyright (C) 2005-2014 by Jordi Sanfeliu <jordi@fibranet.cat>
+# Copyright (C) 2005-2016 by Jordi Sanfeliu <jordi@fibranet.cat>
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -34,7 +34,7 @@ sub logger {
 
 	if(open(OUT, ">> $main::config{httpd_builtin}->{log_file}")) {
 		if($type eq "OK") {
-			print OUT localtime() . " - $type - [$ENV{REMOTE_ADDR}] \"$ENV{REQUEST_METHOD} $url - $ENV{HTTP_USER_AGENT}\"\n";
+			print OUT localtime() . " - $type - [$ENV{REMOTE_ADDR}] \"$ENV{REQUEST_METHOD} $url - " . ($ENV{HTTP_USER_AGENT} || "") . "\"\n";
 		} elsif($type eq "NOTEXIST") {
 			print OUT localtime() . " - $type - [$ENV{REMOTE_ADDR}] File does not exist: $url\n";
 		} elsif($type eq "AUTHERR") {
@@ -55,7 +55,21 @@ sub check_passwd {
 
 	if(open(IN, $main::config{httpd_builtin}->{auth}->{htpasswd})) {
 		while(<IN>) {
-			my %pair = split(':', $_);
+			my $line = trim($_);
+
+			# append character ':' if not exist
+			if(index($line, ":") < 0) {
+				logger("Malformed line in " . $main::config{httpd_builtin}->{auth}->{htpasswd}, "ERROR");
+				$line .= ":" if index($line, ":") < 0;
+			}
+
+			# discard that line if password is missing
+			if(length($line) == index($line, ":") + 1) {
+				logger("Malformed line in " . $main::config{httpd_builtin}->{auth}->{htpasswd}, "ERROR");
+				next;
+			}
+
+			my %pair = split(':', $line);
 			if($pair{$user || ""}) {
 				chomp($pair{$user});
 				if(crypt($pass, $pair{$user}) ne $pair{$user}) {
@@ -127,6 +141,10 @@ sub http_header {
 
 	if($mimetype =~ m/(html|cgi)/) {
 		print "Content-Type: text/html; charset=UTF-8\r\n";
+	} elsif($mimetype eq "css") {
+		print "Content-Type: text/css; charset=UTF-8\r\n";
+	} elsif($mimetype eq "svg") {
+		print "Content-Type: image/svg+xml; charset=UTF-8\r\n";
 	} else {
 		print "Content-Type: image/$mimetype;\r\n";
 	}
@@ -173,7 +191,7 @@ sub handle_request {
 	$denied = ip_validity($ENV{REMOTE_ADDR}, $hosts_deny) if !$allowed;
 	if(!$allowed && $denied) {
 		http_header("403", "html");
-		print "<!DOCTYPE HTML PUBLIC \"-//IETF//DTD HTML 2.0//EN\">\r\n";
+		print("<!DOCTYPE html '-//W3C//DTD HTML 4.01 Final//EN'>\r\n");
 		print "<html><head>\r\n";
 		print "<title>403 Forbidden</title>\r\n";
 		print "</head><body>\r\n";
@@ -203,7 +221,7 @@ sub handle_request {
 	if(!$target || $target eq $base_url) {
 		$target = "index.html" unless $target;
 	}
-	($mimetype) = ($target =~ m/.*\.(html|cgi|png)$/);
+	($mimetype) = ($target =~ m/.*\.(html|cgi|css|png|svg)$/);
 
 	$target =~ s/^\/*//;		# removes leading slashes
 	$target_cgi =~ s/^\/*//;	# removes leading slashes
@@ -226,7 +244,7 @@ sub handle_request {
 	if(scalar(@data)) {
 		if($auth eq "y") {
 			if(http_header("401", $mimetype)) {
-				print "<!DOCTYPE HTML PUBLIC \"-//IETF//DTD HTML 2.0//EN\">\r\n";
+				print("<!DOCTYPE html '-//W3C//DTD HTML 4.01 Final//EN'>\r\n");
 				print "<html><head>\r\n";
 				print "<title>401 Authorization Required</title>\r\n";
 				print "</head><body>\r\n";
@@ -249,7 +267,7 @@ sub handle_request {
 		logger($url, "OK");
 	} else {
 		http_header("404", "html");
-		print "<!DOCTYPE HTML PUBLIC \"-//IETF//DTD HTML 2.0//EN\">\r\n";
+		print("<!DOCTYPE html '-//W3C//DTD HTML 4.01 Final//EN'>\r\n");
 		print "<html><head>\r\n";
 		print "<title>404 Not Found</title>\r\n";
 		print "</head><body>\r\n";
