@@ -1,7 +1,7 @@
 #
 # Monitorix - A lightweight system monitoring tool.
 #
-# Copyright (C) 2005-2014 by Jordi Sanfeliu <jordi@fibranet.cat>
+# Copyright (C) 2005-2017 by Jordi Sanfeliu <jordi@fibranet.cat>
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -32,15 +32,26 @@ sub emailreports_send {
 	my $myself = (caller(0))[3];
 	my ($config, $report, $when, $debug) = @_;
 	my $emailreports = $config->{emailreports};
+	my $imgfmt_lc = lc($config->{image_format});
 
 	my $base_cgi = $config->{base_cgi};
 	my $imgs_dir = $config->{imgs_dir};
 	my $images;
+	my $mime;
 	
 	logger("$myself: sending $report reports.");
 
 	my $uri = URI->new($emailreports->{url_prefix});
-	my $hostname = $uri->host;
+	my $scheme = $uri->scheme || "";
+	my $userinfo = $uri->userinfo || "";
+	$userinfo .= "@" unless !$userinfo;
+	my $hostname = $uri->host || "";
+	my $port = $uri->port || "";
+
+	my $prefix = "$scheme://$userinfo$hostname:$port";
+
+	$mime = "image/png";
+	$mime = "image/svg+xml" if uc($config->{image_format}) eq "SVG";
 
 	my $html = <<"EOF";
 <html>
@@ -71,7 +82,7 @@ EOF
   <a href='http://www.monitorix.org'><img src='cid:image_logo' border='0'></a>
   <br>
   <font face='Verdana, sans-serif' color='000000' size='-2'>
-Copyright &copy; 2005-2014 Jordi Sanfeliu
+Copyright &copy; 2005-2017 Jordi Sanfeliu
   </font>
   </body>
 </html>
@@ -87,8 +98,9 @@ EOF
 			if lc($config->{accept_selfsigned_certs}) eq "y";
 
 		# generate the graphs and get the html source
-		my $url = $emailreports->{url_prefix} . $base_cgi . "/monitorix.cgi?mode=localhost&graph=_$g&when=$when&color=white";
-		my $ua = LWP::UserAgent->new(timeout => 30, $ssl);
+		my $url = $prefix . $base_cgi . "/monitorix.cgi?mode=localhost&graph=_$g&when=$when&color=white";
+		my $ua = LWP::UserAgent->new(timeout => 120, $ssl);
+		$ua->agent($config->{user_agent_id}) if $config->{user_agent_id} || "";
 
 		my $response = $ua->request(HTTP::Request->new('GET', $url));
 
@@ -120,12 +132,12 @@ EOF
 			$n = 1;
 			foreach (split('\n', $graph)) {
 				if(/<img src=/) {
-					push(@tmp, "<img src='cid:image_$g$e$n' border='0'>");
-					$images->{"image_$g$n"} = "";
-
+					push(@tmp, "<img src='cid:image_$g$e$n.$imgfmt_lc' border='0'>");
 					($url) = $_ =~ m/<img src='(.*?)' /;
-					$response = $ua->request(HTTP::Request->new('GET', $url));
-					$images->{"image_$g$e$n"} = $response->content;
+					my $uri = URI->new($url);
+					my $path = $uri->path || "";
+					$response = $ua->request(HTTP::Request->new('GET', "$prefix$path"));
+					$images->{"image_$g$e$n.$imgfmt_lc"} = $response->content;
 					$n++;
 				} else {
 					push(@tmp, $_);
@@ -135,10 +147,15 @@ EOF
 			$html .= join("\n", @tmp);
 			$html .= "<br>";
 
-			# addendum data included in report
-			if($emailreports->{$report}->{addendum_script}) {
-				$html .= <<"EOF";
-<table cellspacing='5' cellpadding='0' width='1' bgcolor='CCCCCC' border='1'>
+			$data =~ s/<!-- graph table begins -->.*?<!-- graph table ends -->//;
+			$e++;
+		}
+	}
+
+	# addendum data included in report
+	if($emailreports->{$report}->{addendum_script}) {
+		$html .= <<"EOF";
+<table cellspacing='5' cellpadding='0' width='100%' bgcolor='CCCCCC' border='1'>
  <tr>
   <td bgcolor='777777'>
    <font face='Verdana, sans-serif' color='CCCC00'>
@@ -149,16 +166,11 @@ EOF
  <tr>
   <td bgcolor='FFFFFF'>
 EOF
-				$html .= `$emailreports->{$report}->{addendum_script}`;
-				$html .= "\n";
-				$html .= "  </td>\n";
-				$html .= " </tr>\n";
-				$html .= "</table>\n";
-			}
-
-			$data =~ s/<!-- graph table begins -->.*?<!-- graph table ends -->//;
-			$e++;
-		}
+		$html .= `$emailreports->{$report}->{addendum_script}`;
+		$html .= "\n";
+		$html .= "  </td>\n";
+		$html .= " </tr>\n";
+		$html .= "</table>\n";
 	}
 
 	$html .= $html_footer;
@@ -186,13 +198,19 @@ EOF
 		);
 		while (my ($key, $val) = each(%{$images})) {
 			$msg->attach(
-				Type		=> 'image/png',
+				Type		=> "$mime; name=\"$key\"",
 				Id		=> $key,
 				Data		=> $val,
 			);
 		}
 
-		$msg->send('smtp', $emailreports->{smtp_hostname}, Timeout => 60);
+		if(lc($emailreports->{method} || "") eq "relay") {
+			$msg->send();
+
+		} else {
+			$msg->send('smtp', $emailreports->{smtp_hostname}, Timeout => 60);
+		}
+
 		logger("\t$myself: to: $to") if $debug;
 	}
 }
