@@ -1,7 +1,7 @@
 #
 # Monitorix - A lightweight system monitoring tool.
 #
-# Copyright (C) 2005-2014 by Jordi Sanfeliu <jordi@fibranet.cat>
+# Copyright (C) 2005-2017 by Jordi Sanfeliu <jordi@fibranet.cat>
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -429,7 +429,7 @@ sub fs_init {
 				# get the major and minor of $d
 				my $rdev = (stat($d))[6];
 				if(!$rdev) {
-					logger("$myself: Unable to detect the device name of '$d'. I/O stats for this filesystem won't be shown in graph.");
+					logger("$myself: Unable to detect the device name of '$f', I/O stats won't be shown in graph. If this is really a mount point then consider using <devmap> to map it manually to a device name.");
 					next;
 				}
 				my $minor = $rdev % 256;
@@ -503,8 +503,14 @@ sub fs_init {
 				}
 			} elsif($config->{os} eq "FreeBSD" || $config->{os} eq "OpenBSD" || $config->{os} eq "NetBSD") {
 				if($f eq "swap") {
-					$d = `swapinfo | tail -1 | awk -F " " '{ print \$1 }'`;
-					chomp($d);
+					if($config->{os} eq "FreeBSD" || $config->{os} eq "NetBSD") {
+						$d = `swapinfo | tail -1 | awk -F " " '{ print \$1 }'`;
+						chomp($d);
+					}
+					if($config->{os} eq "OpenBSD") {
+						$d = `swapctl -l | tail -1 | awk -F " " '{ print \$1 }'`;
+						chomp($d);
+					}
 				}
 
 				# remove the /dev/ prefix
@@ -521,15 +527,12 @@ sub fs_init {
 		}
 	}
 
-	# check dependencies
-	if(lc($fs->{alerts}->{rootfs_enabled}) eq "y") {
-		if(! -x $fs->{alerts}->{rootfs_script}) {
-			logger("$myself: ERROR: script '$fs->{alerts}->{rootfs_script}' doesn't exist or don't has execution permissions.");
-		}
+	# check for deprecated options
+	if($fs->{alerts}->{rootfs_enabled} || $fs->{alerts}->{rootfs_timeintvl} || $fs->{alerts}->{rootfs_threshold} || $fs->{alerts}->{rootfs_script}) {
+		logger("$myself: WARNING: you have deprecated options in the <alerts> section. Please read the monitorix.conf(5) man page and consider also upgrade your current configuration file.");
 	}
 
 	$config->{fs_hist} = ();
-	$config->{fs_hist_alert1} = 0;
 	push(@{$config->{func_update}}, $package);
 	logger("$myself: Ok") if $debug;
 }
@@ -592,7 +595,7 @@ sub fs_update {
 				if($config->{os} eq "Linux") {
 					open(IN, "free |");
 					while(<IN>) {
-						if(/^Swap:\s+\d+\s+(\d+)\s+(\d+)$/) {
+						if(/^Swap:\s+\d+\s+(\d+)\s+(\d+)\s*$/) {
 							$used = $1;
 							$free = $2;
 						}
@@ -623,6 +626,7 @@ sub fs_update {
 				$use = ($used * 100) / ($used + $free) unless $used + $free == 0;
 			} elsif($f) {
 				my $pid;
+				@tmp = (0) x 10;
 				eval {
 					local $SIG{'ALRM'} = sub {
 						if($pid) {
@@ -683,22 +687,27 @@ sub fs_update {
 				# prevents a division by 0 if device is not responding
 				$ino = ($used * 100) / ($used + $free) unless $used + $free == 0;
 
-				# FS alert
-				if($f eq "/" && lc($fs->{alerts}->{rootfs_enabled}) eq "y") {
-					if(!$fs->{alerts}->{rootfs_threshold} || $use < $fs->{alerts}->{rootfs_threshold}) {
-						$config->{fs_hist_alert1} = 0;
+				# check alerts for each filesystem
+				my @al = split(',', $fs->{alerts}->{$f} || "");
+				if(scalar(@al)) {
+					my $timeintvl = trim($al[0]);
+					my $threshold = trim($al[1]);
+					my $script = trim($al[2]);
+
+					if(!$threshold || $use < $threshold) {
+						$config->{fs_hist}->{$f} = 0;
 					} else {
-						if(!$config->{fs_hist_alert1}) {
-							$config->{fs_hist_alert1} = time;
+						if(!$config->{fs_hist}->{$f}) {
+							$config->{fs_hist}->{$f} = time;
 						}
-						if($config->{fs_hist_alert1} > 0 && (time - $config->{fs_hist_alert1}) >= $fs->{alerts}->{rootfs_timeintvl}) {
-							if(-x $fs->{alerts}->{rootfs_script}) {
-								logger("$myself: ALERT: executing script '$fs->{alerts}->{rootfs_script}'.");
-								system($fs->{alerts}->{rootfs_script} . " " . $fs->{alerts}->{rootfs_timeintvl} . " " . $fs->{alerts}->{rootfs_threshold} . " " . $use);
+						if($config->{fs_hist}->{$f} > 0 && (time - $config->{fs_hist}->{$f}) >= $timeintvl) {
+							if(-x $script) {
+								logger("$myself: alert on filesystem '$f': executing script '$script'.");
+								system($script . " " . $timeintvl . " " . $threshold . " " . $use);
 							} else {
-								logger("$myself: ERROR: script '$fs->{alerts}->{rootfs_script}' doesn't exist or don't has execution permissions.");
+								logger("$myself: ERROR: script '$script' doesn't exist or don't has execution permissions.");
 							}
-							$config->{fs_hist_alert1} = time;
+							$config->{fs_hist}->{$f} = time;
 						}
 					}
 				}
@@ -712,14 +721,15 @@ sub fs_update {
 			if($d) {
 				if($config->{os} eq "Linux") {
 					if($config->{kernel} gt "2.4") {
-						open(IN, "/proc/diskstats");
-						while(<IN>) {
-							if(/ $d /) {
-								@tmp = split(' ', $_);
-								last;
+						if(open(IN, "/proc/diskstats")) {
+							while(<IN>) {
+								if(/ $d /) {
+									@tmp = split(' ', $_);
+									last;
+								}
 							}
+							close(IN);
 						}
-						close(IN);
 						(undef, undef, undef, $read_cnt, undef, undef, $read_sec, $write_cnt, undef, undef, $write_sec) = @tmp;
 					} else {
 						my $io;
@@ -760,8 +770,8 @@ sub fs_update {
 				}
 			}
 
-			$ioa = $read_cnt + $write_cnt;
-			$tim = $read_sec + $write_sec;
+			$ioa = ($read_cnt || 0) + ($write_cnt || 0);
+			$tim = ($read_sec || 0) + ($write_sec || 0);
 
 			$str = $e . "_ioa" . $n;
 			$val = $ioa;
@@ -790,23 +800,33 @@ sub fs_update {
 
 sub fs_cgi {
 	my ($package, $config, $cgi) = @_;
+	my @output;
 
 	my $fs = $config->{fs};
-	my @rigid = split(',', $fs->{rigid});
-	my @limit = split(',', $fs->{limit});
+	my @rigid = split(',', ($fs->{rigid} || ""));
+	my @limit = split(',', ($fs->{limit} || ""));
 	my $tf = $cgi->{tf};
 	my $colors = $cgi->{colors};
 	my $graph = $cgi->{graph};
 	my $silent = $cgi->{silent};
 	my $zoom = "--zoom=" . $config->{global_zoom};
+	my %rrd = (
+		'new' => \&RRDs::graphv,
+		'old' => \&RRDs::graph,
+	);
+	my $version = "new";
+	my $pic;
+	my $picz;
+	my $picz_width;
+	my $picz_height;
 
 	my $u = "";
 	my $width;
 	my $height;
 	my $graph_title;
 	my $vlabel;
-	my @PNG;
-	my @PNGz;
+	my @IMG;
+	my @IMGz;
 	my @tmp;
 	my @tmpz;
 	my @CDEF;
@@ -828,9 +848,12 @@ sub fs_cgi {
 		"#EEEE44",
 	);
 
+	$version = "old" if $RRDs::VERSION < 1.3;
 	my $rrd = $config->{base_lib} . $package . ".rrd";
 	my $title = $config->{graph_title}->{$package};
-	my $PNG_DIR = $config->{base_dir} . "/" . $config->{imgs_dir};
+	my $IMG_DIR = $config->{base_dir} . "/" . $config->{imgs_dir};
+	my $imgfmt_uc = uc($config->{image_format});
+	my $imgfmt_lc = lc($config->{image_format});
 
 	$title = !$silent ? $title : "";
 
@@ -839,20 +862,20 @@ sub fs_cgi {
 	#
 	if(lc($config->{iface_mode}) eq "text") {
 		if($title) {
-			main::graph_header($title, 2);
-			print("    <tr>\n");
-			print("    <td bgcolor='$colors->{title_bg_color}'>\n");
+			push(@output, main::graph_header($title, 2));
+			push(@output, "    <tr>\n");
+			push(@output, "    <td bgcolor='$colors->{title_bg_color}'>\n");
 		}
 		my (undef, undef, undef, $data) = RRDs::fetch("$rrd",
 			"--start=-$tf->{nwhen}$tf->{twhen}",
 			"AVERAGE",
 			"-r $tf->{res}");
 		$err = RRDs::error;
-		print("ERROR: while fetching $rrd: $err\n") if $err;
+		push(@output, "ERROR: while fetching $rrd: $err\n") if $err;
 		my $line1;
 		my $line2;
 		my $line3;
-		print("    <pre style='font-size: 12px; color: $colors->{fg_color}';>\n");
+		push(@output, "    <pre style='font-size: 12px; color: $colors->{fg_color}';>\n");
 		foreach my $k (sort keys %{$fs->{list}}) {
 			my @f = split(',', $fs->{list}->{$k});
 			for($n = 0; $n < scalar(@f); $n++) {
@@ -864,9 +887,9 @@ sub fs_cgi {
 				$line3 .=      "-----------------------------";
 			}
 		}
-		print("    $line1\n");
-		print("Time $line2\n");
-		print("-----$line3\n");
+		push(@output, "    $line1\n");
+		push(@output, "Time $line2\n");
+		push(@output, "-----$line3\n");
 		my $line;
 		my @row;
 		my $time;
@@ -876,29 +899,29 @@ sub fs_cgi {
 			$line = @$data[$n];
 			$time = $time - (1 / $tf->{ts});
 			my ($root, $swap) = @$line;
-			printf(" %2d$tf->{tc} ", $time);
+			push(@output, sprintf(" %2d$tf->{tc} ", $time));
 			$e = 0;
 			foreach my $k (sort keys %{$fs->{list}}) {
 				my @f = split(',', $fs->{list}->{$k});
 				for($n2 = 0; $n2 < scalar(@f); $n2++) {
-					$from = ($e * 8 * 5) + ($n2 * 5);
-					$to = $from + 5;
+					$from = ($e * 8 * 8) + ($n2 * 8);
+					$to = $from + 8;
 					my ($use, $ioa, $tim, $ino) = @$line[$from..$to];
 					@row = ($use, $ioa, $tim, $ino);
-					printf(" %4.1f%% %7.1f %7.1f %4.1f%% ", @row);
+					push(@output, sprintf(" %4.1f%% %7.1f %7.1f %4.1f%% ", @row));
 				}
 				$e++;
 			}
-			print("\n");
+			push(@output, "\n");
 		}
-		print("    </pre>\n");
+		push(@output, "    </pre>\n");
 		if($title) {
-			print("    </td>\n");
-			print("    </tr>\n");
-			main::graph_footer();
+			push(@output, "    </td>\n");
+			push(@output, "    </tr>\n");
+			push(@output, main::graph_footer());
 		}
-		print("  <br>\n");
-		return;
+		push(@output, "  <br>\n");
+		return @output;
 	}
 
 
@@ -915,13 +938,13 @@ sub fs_cgi {
 
 	for($n = 0; $n < keys(%{$fs->{list}}); $n++) {
 		for($n2 = 1; $n2 <= 4; $n2++) {
-			$str = $u . $package . $n . $n2 . "." . $tf->{when} . ".png";
-			push(@PNG, $str);
-			unlink("$PNG_DIR" . $str);
+			$str = $u . $package . $n . $n2 . "." . $tf->{when} . ".$imgfmt_lc";
+			push(@IMG, $str);
+			unlink("$IMG_DIR" . $str);
 			if(lc($config->{enable_zoom}) eq "y") {
-				$str = $u . $package . $n . $n2 . "z." . $tf->{when} . ".png";
-				push(@PNGz, $str);
-				unlink("$PNG_DIR" . $str);
+				$str = $u . $package . $n . $n2 . "z." . $tf->{when} . ".$imgfmt_lc";
+				push(@IMGz, $str);
+				unlink("$IMG_DIR" . $str);
 			}
 		}
 	}
@@ -931,21 +954,13 @@ sub fs_cgi {
 		my @f = split(',', $fs->{list}->{$k});
 
 		if($e) {
-			print("   <br>\n");
+			push(@output, "   <br>\n");
 		}
 		if($title) {
-			main::graph_header($title, 2);
+			push(@output, main::graph_header($title, 2));
 		}
 
-		undef(@riglim);
-		if(trim($rigid[0]) eq 1) {
-			push(@riglim, "--upper-limit=" . trim($limit[0]));
-		} else {
-			if(trim($rigid[0]) eq 2) {
-				push(@riglim, "--upper-limit=" . trim($limit[0]));
-				push(@riglim, "--rigid");
-			}
-		}
+		@riglim = @{setup_riglim($rigid[0], $limit[0])};
 		undef(@tmp);
 		undef(@tmpz);
 		undef(@CDEF);
@@ -965,7 +980,7 @@ sub fs_cgi {
 					$color = $LC[$n2++];
 				}
 				push(@tmpz, "LINE2:fs" . $n . $color . ":$str");
-				$str = sprintf("%-23s", $str);
+				$str = sprintf("%-23s", substr($str, 0, 23));
 				push(@tmp, "LINE2:fs" . $n . $color . ":$str");
 				push(@tmp, "GPRINT:fs" . $n . ":LAST:Cur\\: %4.1lf%%");
 				push(@tmp, "GPRINT:fs" . $n . ":MIN: Min\\: %4.1lf%%");
@@ -973,8 +988,8 @@ sub fs_cgi {
 			}
 		}
 		if($title) {
-			print("    <tr>\n");
-			print("    <td bgcolor='$colors->{title_bg_color}'>\n");
+			push(@output, "    <tr>\n");
+			push(@output, "    <td bgcolor='$colors->{title_bg_color}'>\n");
 		}
 		if(lc($config->{show_gaps}) eq "y") {
 			push(@tmp, "AREA:wrongdata#$colors->{gap}:");
@@ -987,16 +1002,14 @@ sub fs_cgi {
 			($width, $height) = split('x', $config->{graph_size}->{main}) if $silent eq "imagetagbig";
 			@tmp = @tmpz;
 		}
-		RRDs::graph("$PNG_DIR" . "$PNG[$e * 4]",
+		$pic = $rrd{$version}->("$IMG_DIR" . "$IMG[$e * 4]",
 			"--title=$config->{graphs}->{_fs1}  ($tf->{nwhen}$tf->{twhen})",
 			"--start=-$tf->{nwhen}$tf->{twhen}",
-			"--imgformat=PNG",
+			"--imgformat=$imgfmt_uc",
 			"--vertical-label=Percent (%)",
 			"--width=$width",
 			"--height=$height",
-			"--upper-limit=100",
 			@riglim,
-			"--lower-limit=0",
 			$zoom,
 			@{$cgi->{version12}},
 			@{$colors->{graph_colors}},
@@ -1012,19 +1025,18 @@ sub fs_cgi {
 			@CDEF,
 			@tmp);
 		$err = RRDs::error;
-		print("ERROR: while graphing $PNG_DIR" . "$PNG[$e * 4]: $err\n") if $err;
+		push(@output, "ERROR: while graphing $IMG_DIR" . "$IMG[$e * 4]: $err\n") if $err;
 		if(lc($config->{enable_zoom}) eq "y") {
 			($width, $height) = split('x', $config->{graph_size}->{zoom});
-			RRDs::graph("$PNG_DIR" . "$PNGz[$e * 4]",
+			$picz = $rrd{$version}->("$IMG_DIR" . "$IMGz[$e * 4]",
 				"--title=$config->{graphs}->{_fs1}  ($tf->{nwhen}$tf->{twhen})",
 				"--start=-$tf->{nwhen}$tf->{twhen}",
-				"--imgformat=PNG",
+				"--imgformat=$imgfmt_uc",
 				"--vertical-label=Percent (%)",
 				"--width=$width",
 				"--height=$height",
-				"--upper-limit=100",
 				@riglim,
-				"--lower-limit=0",
+				$zoom,
 				@{$cgi->{version12}},
 				@{$colors->{graph_colors}},
 				"DEF:fs0=$rrd:fs" . $e . "_use0:AVERAGE",
@@ -1039,35 +1051,33 @@ sub fs_cgi {
 				@CDEF,
 				@tmpz);
 			$err = RRDs::error;
-			print("ERROR: while graphing $PNG_DIR" . "$PNGz[$e * 4]: $err\n") if $err;
+			push(@output, "ERROR: while graphing $IMG_DIR" . "$IMGz[$e * 4]: $err\n") if $err;
 		}
 		$e2 = $e . "1";
 		if($title || ($silent =~ /imagetag/ && $graph =~ /fs$e2/)) {
 			if(lc($config->{enable_zoom}) eq "y") {
 				if(lc($config->{disable_javascript_void}) eq "y") {
-					print("      <a href=\"" . $config->{url} . "/" . $config->{imgs_dir} . $PNGz[$e * 4] . "\"><img src='" . $config->{url} . "/" . $config->{imgs_dir} . $PNG[$e * 4] . "' border='0'></a>\n");
-				}
-				else {
-					print("      <a href=\"javascript:void(window.open('" . $config->{url} . "/" . $config->{imgs_dir} . $PNGz[$e * 4] . "','','width=" . ($width + 115) . ",height=" . ($height + 100) . ",scrollbars=0,resizable=0'))\"><img src='" . $config->{url} . "/" . $config->{imgs_dir} . $PNG[$e * 4] . "' border='0'></a>\n");
+					push(@output, "      <a href=\"" . $config->{url} . "/" . $config->{imgs_dir} . $IMGz[$e * 4] . "\"><img src='" . $config->{url} . "/" . $config->{imgs_dir} . $IMG[$e * 4] . "' border='0'></a>\n");
+				} else {
+					if($version eq "new") {
+						$picz_width = $picz->{image_width} * $config->{global_zoom};
+						$picz_height = $picz->{image_height} * $config->{global_zoom};
+					} else {
+						$picz_width = $width + 115;
+						$picz_height = $height + 100;
+					}
+					push(@output, "      <a href=\"javascript:void(window.open('" . $config->{url} . "/" . $config->{imgs_dir} . $IMGz[$e * 4] . "','','width=" . $picz_width . ",height=" . $picz_height . ",scrollbars=0,resizable=0'))\"><img src='" . $config->{url} . "/" . $config->{imgs_dir} . $IMG[$e * 4] . "' border='0'></a>\n");
 				}
 			} else {
-				print("      <img src='" . $config->{url} . "/" . $config->{imgs_dir} . $PNG[$e * 4] . "'>\n");
+				push(@output, "      <img src='" . $config->{url} . "/" . $config->{imgs_dir} . $IMG[$e * 4] . "'>\n");
 			}
 		}
 
 		if($title) {
-			print("    </td>\n");
-			print("    <td valign='top' bgcolor='" . $colors->{title_bg_color} . "'>\n");
-		}
-		undef(@riglim);
-		if(trim($rigid[1]) eq 1) {
-			push(@riglim, "--upper-limit=" . trim($limit[1]));
-		} else {
-			if(trim($rigid[1]) eq 2) {
-				push(@riglim, "--upper-limit=" . trim($limit[1]));
-				push(@riglim, "--rigid");
-			}
+			push(@output, "    </td>\n");
+			push(@output, "    <td valign='top' bgcolor='" . $colors->{title_bg_color} . "'>\n");
 		}
+		@riglim = @{setup_riglim($rigid[1], $limit[1])};
 		undef(@tmp);
 		undef(@tmpz);
 		undef(@CDEF);
@@ -1087,7 +1097,7 @@ sub fs_cgi {
 					$color = $LC[$n2++];
 				}
 				push(@tmpz, "LINE2:ioa" . $n . $color . ":$str");
-				$str = sprintf("%-23s", $str);
+				$str = sprintf("%-23s", substr($str, 0, 23));
 				push(@tmp, "LINE2:ioa" . $n . $color . ":$str");
 				push(@tmp, "GPRINT:ioa" . $n . ":LAST:Cur\\: %4.0lf");
 				push(@tmp, "GPRINT:ioa" . $n . ":MIN: Min\\: %4.0lf");
@@ -1108,15 +1118,14 @@ sub fs_cgi {
 			push(@tmp, "COMMENT: \\n");
 			push(@tmp, "COMMENT: \\n");
 		}
-		RRDs::graph("$PNG_DIR" . "$PNG[$e * 4 + 1]",
+		$pic = $rrd{$version}->("$IMG_DIR" . "$IMG[$e * 4 + 1]",
 			"--title=$config->{graphs}->{_fs2}  ($tf->{nwhen}$tf->{twhen})",
 			"--start=-$tf->{nwhen}$tf->{twhen}",
-			"--imgformat=PNG",
+			"--imgformat=$imgfmt_uc",
 			"--vertical-label=Reads+Writes/s",
 			"--width=$width",
 			"--height=$height",
 			@riglim,
-			"--lower-limit=0",
 			$zoom,
 			@{$cgi->{version12}},
 			@{$colors->{graph_colors}},
@@ -1132,18 +1141,18 @@ sub fs_cgi {
 			@CDEF,
 			@tmp);
 		$err = RRDs::error;
-		print("ERROR: while graphing $PNG_DIR" . "$PNG[$e * 4 + 1]: $err\n") if $err;
+		push(@output, "ERROR: while graphing $IMG_DIR" . "$IMG[$e * 4 + 1]: $err\n") if $err;
 		if(lc($config->{enable_zoom}) eq "y") {
 			($width, $height) = split('x', $config->{graph_size}->{zoom});
-			RRDs::graph("$PNG_DIR" . "$PNGz[$e * 4 + 1]",
+			$picz = $rrd{$version}->("$IMG_DIR" . "$IMGz[$e * 4 + 1]",
 				"--title=$config->{graphs}->{_fs2}  ($tf->{nwhen}$tf->{twhen})",
 				"--start=-$tf->{nwhen}$tf->{twhen}",
-				"--imgformat=PNG",
+				"--imgformat=$imgfmt_uc",
 				"--vertical-label=Reads+Writes/s",
 				"--width=$width",
 				"--height=$height",
 				@riglim,
-				"--lower-limit=0",
+				$zoom,
 				@{$cgi->{version12}},
 				@{$colors->{graph_colors}},
 				"DEF:ioa0=$rrd:fs" . $e . "_ioa0:AVERAGE",
@@ -1158,31 +1167,29 @@ sub fs_cgi {
 				@CDEF,
 				@tmpz);
 			$err = RRDs::error;
-			print("ERROR: while graphing $PNG_DIR" . "$PNGz[$e * 4 + 1]: $err\n") if $err;
+			push(@output, "ERROR: while graphing $IMG_DIR" . "$IMGz[$e * 4 + 1]: $err\n") if $err;
 		}
 		$e2 = $e . "2";
 		if($title || ($silent =~ /imagetag/ && $graph =~ /fs$e2/)) {
 			if(lc($config->{enable_zoom}) eq "y") {
 				if(lc($config->{disable_javascript_void}) eq "y") {
-					print("      <a href=\"" . $config->{url} . "/" . $config->{imgs_dir} . $PNGz[$e * 4 + 1] . "\"><img src='" . $config->{url} . "/" . $config->{imgs_dir} . $PNG[$e * 4 + 1] . "' border='0'></a>\n");
-				}
-				else {
-					print("      <a href=\"javascript:void(window.open('" . $config->{url} . "/" . $config->{imgs_dir} . $PNGz[$e * 4 + 1] . "','','width=" . ($width + 115) . ",height=" . ($height + 100) . ",scrollbars=0,resizable=0'))\"><img src='" . $config->{url} . "/" . $config->{imgs_dir} . $PNG[$e * 4 + 1] . "' border='0'></a>\n");
+					push(@output, "      <a href=\"" . $config->{url} . "/" . $config->{imgs_dir} . $IMGz[$e * 4 + 1] . "\"><img src='" . $config->{url} . "/" . $config->{imgs_dir} . $IMG[$e * 4 + 1] . "' border='0'></a>\n");
+				} else {
+					if($version eq "new") {
+						$picz_width = $picz->{image_width} * $config->{global_zoom};
+						$picz_height = $picz->{image_height} * $config->{global_zoom};
+					} else {
+						$picz_width = $width + 115;
+						$picz_height = $height + 100;
+					}
+					push(@output, "      <a href=\"javascript:void(window.open('" . $config->{url} . "/" . $config->{imgs_dir} . $IMGz[$e * 4 + 1] . "','','width=" . $picz_width . ",height=" . $picz_height . ",scrollbars=0,resizable=0'))\"><img src='" . $config->{url} . "/" . $config->{imgs_dir} . $IMG[$e * 4 + 1] . "' border='0'></a>\n");
 				}
 			} else {
-				print("      <img src='" . $config->{url} . "/" . $config->{imgs_dir} . $PNG[$e * 4 + 1] . "'>\n");
+				push(@output, "      <img src='" . $config->{url} . "/" . $config->{imgs_dir} . $IMG[$e * 4 + 1] . "'>\n");
 			}
 		}
 
-		undef(@riglim);
-		if(trim($rigid[2]) eq 1) {
-			push(@riglim, "--upper-limit=" . trim($limit[2]));
-		} else {
-			if(trim($rigid[2]) eq 2) {
-				push(@riglim, "--upper-limit=" . trim($limit[2]));
-				push(@riglim, "--rigid");
-			}
-		}
+		@riglim = @{setup_riglim($rigid[2], $limit[2])};
 		undef(@tmp);
 		undef(@tmpz);
 		undef(@CDEF);
@@ -1202,7 +1209,7 @@ sub fs_cgi {
 					$color = $LC[$n2++];
 				}
 				push(@tmpz, "LINE2:fs" . $n . $color . ":$str");
-				$str = sprintf("%-23s", $str);
+				$str = sprintf("%-23s", substr($str, 0, 23));
 				push(@tmp, "LINE2:fs" . $n . $color . ":$str");
 				push(@tmp, "GPRINT:fs" . $n . ":LAST:Cur\\: %4.1lf%%");
 				push(@tmp, "GPRINT:fs" . $n . ":MIN: Min\\: %4.1lf%%");
@@ -1210,8 +1217,8 @@ sub fs_cgi {
 			}
 		}
 		if($title) {
-			print("    <tr>\n");
-			print("    <td bgcolor='$colors->{title_bg_color}'>\n");
+			push(@output, "    <tr>\n");
+			push(@output, "    <td bgcolor='$colors->{title_bg_color}'>\n");
 		}
 		if(lc($config->{show_gaps}) eq "y") {
 			push(@tmp, "AREA:wrongdata#$colors->{gap}:");
@@ -1224,16 +1231,14 @@ sub fs_cgi {
 			($width, $height) = split('x', $config->{graph_size}->{main}) if $silent eq "imagetagbig";
 			@tmp = @tmpz;
 		}
-		RRDs::graph("$PNG_DIR" . "$PNG[$e * 4 + 2]",
+		$pic = $rrd{$version}->("$IMG_DIR" . "$IMG[$e * 4 + 2]",
 			"--title=$config->{graphs}->{_fs3}  ($tf->{nwhen}$tf->{twhen})",
 			"--start=-$tf->{nwhen}$tf->{twhen}",
-			"--imgformat=PNG",
+			"--imgformat=$imgfmt_uc",
 			"--vertical-label=Percent (%)",
 			"--width=$width",
 			"--height=$height",
-			"--upper-limit=100",
 			@riglim,
-			"--lower-limit=0",
 			$zoom,
 			@{$cgi->{version12}},
 			@{$colors->{graph_colors}},
@@ -1249,19 +1254,18 @@ sub fs_cgi {
 			@CDEF,
 			@tmp);
 		$err = RRDs::error;
-		print("ERROR: while graphing $PNG_DIR" . "$PNG[$e * 4 + 2]: $err\n") if $err;
+		push(@output, "ERROR: while graphing $IMG_DIR" . "$IMG[$e * 4 + 2]: $err\n") if $err;
 		if(lc($config->{enable_zoom}) eq "y") {
 			($width, $height) = split('x', $config->{graph_size}->{zoom});
-			RRDs::graph("$PNG_DIR" . "$PNGz[$e * 4 + 2]",
+			$picz = $rrd{$version}->("$IMG_DIR" . "$IMGz[$e * 4 + 2]",
 				"--title=$config->{graphs}->{_fs3}  ($tf->{nwhen}$tf->{twhen})",
 				"--start=-$tf->{nwhen}$tf->{twhen}",
-				"--imgformat=PNG",
+				"--imgformat=$imgfmt_uc",
 				"--vertical-label=Percent (%)",
 				"--width=$width",
 				"--height=$height",
-				"--upper-limit=100",
 				@riglim,
-				"--lower-limit=0",
+				$zoom,
 				@{$cgi->{version12}},
 				@{$colors->{graph_colors}},
 				"DEF:fs0=$rrd:fs" . $e . "_ino0:AVERAGE",
@@ -1276,35 +1280,33 @@ sub fs_cgi {
 				@CDEF,
 				@tmpz);
 			$err = RRDs::error;
-			print("ERROR: while graphing $PNG_DIR" . "$PNGz[$e * 4 + 2]: $err\n") if $err;
+			push(@output, "ERROR: while graphing $IMG_DIR" . "$IMGz[$e * 4 + 2]: $err\n") if $err;
 		}
 		$e2 = $e . "3";
 		if($title || ($silent =~ /imagetag/ && $graph =~ /fs$e2/)) {
 			if(lc($config->{enable_zoom}) eq "y") {
 				if(lc($config->{disable_javascript_void}) eq "y") {
-					print("      <a href=\"" . $config->{url} . "/" . $config->{imgs_dir} . $PNGz[$e * 4 + 2] . "\"><img src='" . $config->{url} . "/" . $config->{imgs_dir} . $PNG[$e * 4 + 2] . "' border='0'></a>\n");
-				}
-				else {
-					print("      <a href=\"javascript:void(window.open('" . $config->{url} . "/" . $config->{imgs_dir} . $PNGz[$e * 4 + 2] . "','','width=" . ($width + 115) . ",height=" . ($height + 100) . ",scrollbars=0,resizable=0'))\"><img src='" . $config->{url} . "/" . $config->{imgs_dir} . $PNG[$e * 4 + 2] . "' border='0'></a>\n");
+					push(@output, "      <a href=\"" . $config->{url} . "/" . $config->{imgs_dir} . $IMGz[$e * 4 + 2] . "\"><img src='" . $config->{url} . "/" . $config->{imgs_dir} . $IMG[$e * 4 + 2] . "' border='0'></a>\n");
+				} else {
+					if($version eq "new") {
+						$picz_width = $picz->{image_width} * $config->{global_zoom};
+						$picz_height = $picz->{image_height} * $config->{global_zoom};
+					} else {
+						$picz_width = $width + 115;
+						$picz_height = $height + 100;
+					}
+					push(@output, "      <a href=\"javascript:void(window.open('" . $config->{url} . "/" . $config->{imgs_dir} . $IMGz[$e * 4 + 2] . "','','width=" . $picz_width . ",height=" . $picz_height . ",scrollbars=0,resizable=0'))\"><img src='" . $config->{url} . "/" . $config->{imgs_dir} . $IMG[$e * 4 + 2] . "' border='0'></a>\n");
 				}
 			} else {
-				print("      <img src='" . $config->{url} . "/" . $config->{imgs_dir} . $PNG[$e * 4 + 2] . "'>\n");
+				push(@output, "      <img src='" . $config->{url} . "/" . $config->{imgs_dir} . $IMG[$e * 4 + 2] . "'>\n");
 			}
 		}
 
 		if($title) {
-			print("    </td>\n");
-			print("    <td valign='top' bgcolor='" . $colors->{title_bg_color} . "'>\n");
-		}
-		undef(@riglim);
-		if(trim($rigid[3]) eq 1) {
-			push(@riglim, "--upper-limit=" . trim($limit[3]));
-		} else {
-			if(trim($rigid[3]) eq 2) {
-				push(@riglim, "--upper-limit=" . trim($limit[3]));
-				push(@riglim, "--rigid");
-			}
+			push(@output, "    </td>\n");
+			push(@output, "    <td valign='top' bgcolor='" . $colors->{title_bg_color} . "'>\n");
 		}
+		@riglim = @{setup_riglim($rigid[3], $limit[3])};
 		undef(@tmp);
 		undef(@tmpz);
 		undef(@CDEF);
@@ -1332,7 +1334,7 @@ sub fs_cgi {
 						$color = $LC[$n2++];
 					}
 					push(@tmpz, "LINE2:tim" . $n . $color . ":$str");
-					$str = sprintf("%-23s", $str);
+					$str = sprintf("%-23s", substr($str, 0, 23));
 					push(@tmp, "LINE2:tim" . $n . $color . ":$str");
 					push(@tmp, "GPRINT:stim" . $n . ":LAST:Cur\\: %4.1lfs");
 					push(@tmp, "GPRINT:stim" . $n . ":MIN:Min\\: %4.1lfs");
@@ -1358,7 +1360,7 @@ sub fs_cgi {
 						$color = $LC[$n2++];
 					}
 					push(@tmpz, "LINE2:tim" . $n . $color . ":$str");
-					$str = sprintf("%-23s", $str);
+					$str = sprintf("%-23s", substr($str, 0, 23));
 					push(@tmp, "LINE2:tim" . $n . $color . ":$str");
 					push(@tmp, "GPRINT:tim" . $n . ":LAST:Cur\\: %4.0lf");
 					push(@tmp, "GPRINT:tim" . $n . ":MIN: Min\\: %4.0lf");
@@ -1380,15 +1382,14 @@ sub fs_cgi {
 			push(@tmp, "COMMENT: \\n");
 			push(@tmp, "COMMENT: \\n");
 		}
-		RRDs::graph("$PNG_DIR" . "$PNG[$e * 4 + 3]",
+		$pic = $rrd{$version}->("$IMG_DIR" . "$IMG[$e * 4 + 3]",
 			"--title=$graph_title",
 			"--start=-$tf->{nwhen}$tf->{twhen}",
-			"--imgformat=PNG",
+			"--imgformat=$imgfmt_uc",
 			"--vertical-label=$vlabel",
 			"--width=$width",
 			"--height=$height",
 			@riglim,
-			"--lower-limit=0",
 			$zoom,
 			@{$cgi->{version12}},
 			@{$colors->{graph_colors}},
@@ -1412,18 +1413,18 @@ sub fs_cgi {
 			@CDEF,
 			@tmp);
 		$err = RRDs::error;
-		print("ERROR: while graphing $PNG_DIR" . "$PNG[$e * 4 + 3]: $err\n") if $err;
+		push(@output, "ERROR: while graphing $IMG_DIR" . "$IMG[$e * 4 + 3]: $err\n") if $err;
 		if(lc($config->{enable_zoom}) eq "y") {
 			($width, $height) = split('x', $config->{graph_size}->{zoom});
-			RRDs::graph("$PNG_DIR" . "$PNGz[$e * 4 + 3]",
+			$picz = $rrd{$version}->("$IMG_DIR" . "$IMGz[$e * 4 + 3]",
 				"--title=$graph_title",
 				"--start=-$tf->{nwhen}$tf->{twhen}",
-				"--imgformat=PNG",
+				"--imgformat=$imgfmt_uc",
 				"--vertical-label=$vlabel",
 				"--width=$width",
 				"--height=$height",
 				@riglim,
-				"--lower-limit=0",
+				$zoom,
 				@{$cgi->{version12}},
 				@{$colors->{graph_colors}},
 				"DEF:tim0=$rrd:fs" . $e . "_tim0:AVERAGE",
@@ -1446,31 +1447,37 @@ sub fs_cgi {
 				@CDEF,
 				@tmpz);
 			$err = RRDs::error;
-			print("ERROR: while graphing $PNG_DIR" . "$PNGz[$e * 4 + 3]: $err\n") if $err;
+			push(@output, "ERROR: while graphing $IMG_DIR" . "$IMGz[$e * 4 + 3]: $err\n") if $err;
 		}
 		$e2 = $e . "4";
 		if($title || ($silent =~ /imagetag/ && $graph =~ /fs$e2/)) {
 			if(lc($config->{enable_zoom}) eq "y") {
 				if(lc($config->{disable_javascript_void}) eq "y") {
-					print("      <a href=\"" . $config->{url} . "/" . $config->{imgs_dir} . $PNGz[$e * 4 + 3] . "\"><img src='" . $config->{url} . "/" . $config->{imgs_dir} . $PNG[$e * 4 + 3] . "' border='0'></a>\n");
-				}
-				else {
-					print("      <a href=\"javascript:void(window.open('" . $config->{url} . "/" . $config->{imgs_dir} . $PNGz[$e * 4 + 3] . "','','width=" . ($width + 115) . ",height=" . ($height + 100) . ",scrollbars=0,resizable=0'))\"><img src='" . $config->{url} . "/" . $config->{imgs_dir} . $PNG[$e * 4 + 3] . "' border='0'></a>\n");
+					push(@output, "      <a href=\"" . $config->{url} . "/" . $config->{imgs_dir} . $IMGz[$e * 4 + 3] . "\"><img src='" . $config->{url} . "/" . $config->{imgs_dir} . $IMG[$e * 4 + 3] . "' border='0'></a>\n");
+				} else {
+					if($version eq "new") {
+						$picz_width = $picz->{image_width} * $config->{global_zoom};
+						$picz_height = $picz->{image_height} * $config->{global_zoom};
+					} else {
+						$picz_width = $width + 115;
+						$picz_height = $height + 100;
+					}
+					push(@output, "      <a href=\"javascript:void(window.open('" . $config->{url} . "/" . $config->{imgs_dir} . $IMGz[$e * 4 + 3] . "','','width=" . $picz_width . ",height=" . $picz_height . ",scrollbars=0,resizable=0'))\"><img src='" . $config->{url} . "/" . $config->{imgs_dir} . $IMG[$e * 4 + 3] . "' border='0'></a>\n");
 				}
 			} else {
-				print("      <img src='" . $config->{url} . "/" . $config->{imgs_dir} . $PNG[$e * 4 + 3] . "'>\n");
+				push(@output, "      <img src='" . $config->{url} . "/" . $config->{imgs_dir} . $IMG[$e * 4 + 3] . "'>\n");
 			}
 		}
 
 		if($title) {
-			print("    </td>\n");
-			print("    </tr>\n");
-			main::graph_footer();
+			push(@output, "    </td>\n");
+			push(@output, "    </tr>\n");
+			push(@output, main::graph_footer());
 		}
 		$e++;
 	}
-	print("  <br>\n");
-	return;
+	push(@output, "  <br>\n");
+	return @output;
 }
 
 1;
