@@ -1,7 +1,7 @@
 #
 # Monitorix - A lightweight system monitoring tool.
 #
-# Copyright (C) 2005-2014 by Jordi Sanfeliu <jordi@fibranet.cat>
+# Copyright (C) 2005-2017 by Jordi Sanfeliu <jordi@fibranet.cat>
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -88,8 +88,8 @@ sub system_init {
 				"DS:system_val01:GAUGE:120:0:U",
 				"DS:system_val02:GAUGE:120:0:U",
 				"DS:system_val03:GAUGE:120:0:U",
-				"DS:system_val04:GAUGE:120:0:U",
-				"DS:system_val05:GAUGE:120:0:U",
+				"DS:system_entrop:GAUGE:120:0:U",
+				"DS:system_uptime:GAUGE:120:0:U",
 				"RRA:AVERAGE:0.5:1:1440",
 				"RRA:AVERAGE:0.5:30:336",
 				"RRA:AVERAGE:0.5:60:744",
@@ -128,6 +128,12 @@ sub system_init {
 		}
 	}
 
+	# Since 3.10.0 two new values were included (entropy and uptime)
+	RRDs::tune($rrd,
+		"--data-source-rename=system_val04:system_entrop",
+		"--data-source-rename=system_val05:system_uptime",
+	);
+
 	$config->{system_hist_alert1} = 0;
 	push(@{$config->{func_update}}, $package);
 	logger("$myself: Ok") if $debug;
@@ -158,26 +164,48 @@ sub system_update {
 	my $val01 = 0;
 	my $val02 = 0;
 	my $val03 = 0;
-	my $val04 = 0;
-	my $val05 = 0;
+	my $entropy = 0;
+	my $uptime = 0;
 
 	my $rrdata = "N";
 
-	$npwio = $npzom = $npstp = $npswp = 0;
-
 	if($config->{os} eq "Linux") {
+		my $dir;
+
 		open(IN, "/proc/loadavg");
 		while(<IN>) {
-			if(/^(\d+\.\d+) (\d+\.\d+) (\d+\.\d+) (\d+)\/(\d+)/) {
+			if(/^(\d+\.\d+) (\d+\.\d+) (\d+\.\d+) /) {
 				$load1 = $1;
 				$load5 = $2;
 				$load15 = $3;
-				$nprun = $4;
-				$npslp = $5;
 			}
 		}
 		close(IN);
-		$nproc = $npslp + $nprun;
+
+		foreach $dir (</proc/[0-9]*>) {
+			if(-d $dir) {
+				my $status = $dir . "/status";
+				if(-f $status) {
+					open(IN, $status);
+						while(<IN>) {
+							if(/^State:/) {
+								my @tmp = split(' ', $_);
+								my (undef, $state) = @tmp;
+								$nprun++ if $state eq "R";
+								$npslp++ if $state eq "S";
+								$npwio++ if $state eq "D";
+								$npzom++ if $state eq "Z";
+								$npstp++ if $state eq "T";
+								$npswp++ if $state eq "W";
+								last;
+							}
+						}
+					close(IN);
+				}
+			}
+		}
+
+		$nproc = $npslp + $nprun + $npwio + $npzom + $npstp + $npswp;
 
 		open(IN, "/proc/meminfo");
 		while(<IN>) {
@@ -195,11 +223,34 @@ sub system_update {
 			}
 			if(/^Cached:\s+(\d+) kB$/) {
 				$mcach = $1;
+				next;
+			}
+			if(/^Active:\s+(\d+) kB$/) {
+				$macti = $1;
+				next;
+			}
+			if(/^Inactive:\s+(\d+) kB$/) {
+				$minac = $1;
 				last;
 			}
 		}
 		close(IN);
-		$macti = $minac = "";
+
+		open(IN, "/proc/sys/kernel/random/entropy_avail");
+		while(<IN>) {
+			if(/^(\d+)$/) {
+				$entropy = $1;
+			}
+		}
+		close(IN);
+
+		open(IN, "/proc/uptime");
+		while(<IN>) {
+			if(/^(\d+)\./) {
+				$uptime = $1;
+			}
+		}
+		close(IN);
 	} elsif($config->{os} eq "FreeBSD") {
 		my $page_size;
 		open(IN, "sysctl -n vm.loadavg |");
@@ -238,6 +289,17 @@ sub system_update {
 		$mcach = ($page_size * $mcach) / 1024;
 		$macti = ($page_size * $macti) / 1024;
 		$minac = ($page_size * $minac) / 1024;
+
+		open(IN, "/sbin/sysctl -n kern.random.sys.seeded |");
+		$entropy = <IN>;
+		close(IN);
+		chomp($entropy);
+
+		open(IN, "/sbin/sysctl -n kern.boottime |");
+		(undef, undef, undef, $uptime) = split(' ', <IN>);
+		close(IN);
+		$uptime =~ s/,//;
+		$uptime = time - int($uptime);
 	} elsif($config->{os} eq "OpenBSD" || $config->{os} eq "NetBSD") {
 		open(IN, "sysctl -n vm.loadavg |");
 		while(<IN>) {
@@ -265,7 +327,7 @@ sub system_update {
 				}
 			}
 			if($config->{os} eq "OpenBSD") {
-				if(/^Memory: Real: (\d+)\w\/\d+\w act\/tot  Free: (\d+)\w  /) {
+				if(/^Memory:\s+Real:\s+(\d+)\w\/\d+\w\s+act\/tot\s+Free:\s+(\d+)\w\s+/) {
 					$macti = $1;
 					$mfree = $2;
 					$macti = int($macti) * 1024;
@@ -287,6 +349,12 @@ sub system_update {
 		$mtotl = `sysctl -n hw.physmem`;
 		chomp($mtotl);
 		$mtotl = $mtotl / 1024;
+
+		open(IN, "/sbin/sysctl -n kern.boottime |");
+		$uptime = <IN>;
+		close(IN);
+		chomp($uptime);
+		$uptime = time - int($uptime);
 	}
 
 	chomp(
@@ -306,6 +374,8 @@ sub system_update {
 		$mfree,
 		$macti,
 		$minac,
+		$entropy,
+		$uptime,
 	);
 
 	# SYSTEM alert
@@ -328,7 +398,7 @@ sub system_update {
 		}
 	}
 
-	$rrdata .= ":$load1:$load5:$load15:$nproc:$npslp:$nprun:$npwio:$npzom:$npstp:$npswp:$mtotl:$mbuff:$mcach:$mfree:$macti:$minac:$val01:$val02:$val03:$val04:$val05";
+	$rrdata .= ":$load1:$load5:$load15:$nproc:$npslp:$nprun:$npwio:$npzom:$npstp:$npswp:$mtotl:$mbuff:$mcach:$mfree:$macti:$minac:$val01:$val02:$val03:$entropy:$uptime";
 	RRDs::update($rrd, $rrdata);
 	logger("$myself: $rrdata") if $debug;
 	my $err = RRDs::error;
@@ -337,15 +407,25 @@ sub system_update {
 
 sub system_cgi {
 	my ($package, $config, $cgi) = @_;
+	my @output;
 
 	my $system = $config->{system};
-	my @rigid = split(',', $system->{rigid});
-	my @limit = split(',', $system->{limit});
+	my @rigid = split(',', ($system->{rigid} || ""));
+	my @limit = split(',', ($system->{limit} || ""));
 	my $tf = $cgi->{tf};
 	my $colors = $cgi->{colors};
 	my $graph = $cgi->{graph};
 	my $silent = $cgi->{silent};
 	my $zoom = "--zoom=" . $config->{global_zoom};
+	my %rrd = (
+		'new' => \&RRDs::graphv,
+		'old' => \&RRDs::graph,
+	);
+	my $version = "new";
+	my $pic;
+	my $picz;
+	my $picz_width;
+	my $picz_height;
 
 	my $u = "";
 	my $width;
@@ -357,13 +437,18 @@ sub system_cgi {
 	my $n;
 	my $err;
 
+	$version = "old" if $RRDs::VERSION < 1.3;
 	my $rrd = $config->{base_lib} . $package . ".rrd";
 	my $title = $config->{graph_title}->{$package};
-	my $PNG_DIR = $config->{base_dir} . "/" . $config->{imgs_dir};
+	my $IMG_DIR = $config->{base_dir} . "/" . $config->{imgs_dir};
+	my $imgfmt_uc = uc($config->{image_format});
+	my $imgfmt_lc = lc($config->{image_format});
+
 
 	$title = !$silent ? $title : "";
 
 	my $total_mem;
+	my $total_mem_bytes;
 
 	if($config->{os} eq "Linux") {
 		$total_mem = `grep -w MemTotal: /proc/meminfo | awk '{print \$2}'`;
@@ -373,6 +458,7 @@ sub system_cgi {
 		chomp($total_mem);
 		$total_mem = int($total_mem / 1024);		# in KB
 	}
+	$total_mem_bytes = int($total_mem * 1024);		# in bytes
 	$total_mem = int($total_mem / 1024);			# in MB
 
 
@@ -380,43 +466,44 @@ sub system_cgi {
 	#
 	if(lc($config->{iface_mode}) eq "text") {
 		if($title) {
-			main::graph_header($title, 2);
-			print("    <tr>\n");
-			print("    <td bgcolor='$colors->{title_bg_color}'>\n");
+			push(@output, main::graph_header($title, 2));
+			push(@output, "    <tr>\n");
+			push(@output, "    <td bgcolor='$colors->{title_bg_color}'>\n");
 		}
 		my (undef, undef, undef, $data) = RRDs::fetch($rrd,
 			"--start=-$tf->{nwhen}$tf->{twhen}",
 			"AVERAGE",
 			"-r $tf->{res}");
 		$err = RRDs::error;
-		print("ERROR: while fetching $rrd: $err\n") if $err;
-		print("    <pre style='font-size: 12px; color: $colors->{fg_color}';>\n");
-		print("       CPU load average    Memory usage in MB     Processes\n");
-		print("Time   1min  5min 15min    Used  Cached Buffers   Total   Run\n");
-		print("------------------------------------------------------------- \n");
+		push(@output, "ERROR: while fetching $rrd: $err\n") if $err;
+		push(@output, "    <pre style='font-size: 12px; color: $colors->{fg_color}';>\n");
+		push(@output, "       CPU load average                      Memory usage in MB                           Processes\n");
+		push(@output, "Time   1min  5min 15min    Used  Cached Buffers  Active Inactiv  Total   R    S    D    Z    T    W Entropy   Uptime\n");
+		push(@output, "-------------------------------------------------------------------------------------------------------------------- \n");
 		my $line;
 		my @row;
 		my $time;
 		for($n = 0, $time = $tf->{tb}; $n < ($tf->{tb} * $tf->{ts}); $n++) {
 			$line = @$data[$n];
-			my ($load1, $load5, $load15, $nproc, $npslp, $nprun, $mtotl, $buff, $cach, $free) = @$line;
+			my ($load1, $load5, $load15, $nproc, $npslp, $nprun, $npwio, $npzom, $npstp, $npswp, $mtotl, $buff, $cach, $free, $macti, $minac, $val01, $val02, $val03, $entropy, $uptime) = @$line;
 			$buff /= 1024;
 			$cach /= 1024;
 			$free /= 1024;
-			@row = ($load1, $load5, $load15, $total_mem - $free, $cach, $buff, $nproc, $nprun);
+			@row = ($load1, $load5, $load15, $total_mem - $free, $cach, $buff, $macti, $minac, $nproc, $nprun, $npslp, $npwio, $npzom, $npstp, $npswp, $entropy, $uptime);
 			$time = $time - (1 / $tf->{ts});
-			printf(" %2d$tf->{tc}   %4.1f  %4.1f  %4.1f  %6d  %6d  %6d   %5d %5d \n", $time, @row);
+			push(@output, sprintf(" %2d$tf->{tc}   %4.1f  %4.1f  %4.1f  %6d  %6d  %6d  %6d  %6d  %4d %4d %4d %4d %4d %4d %4d  %6d %8d\n", $time, @row));
 		}
-		print("\n");
-		print(" system uptime: " . get_uptime($config) . "\n");
-		print("    </pre>\n");
+		push(@output, "\n");
+		push(@output, " system uptime: " . get_uptime($config) . "\n");
+		push(@output, "    </pre>\n");
 		if($title) {
-			print("    </td>\n");
-			print("    </tr>\n");
-			main::graph_footer();
+			my @footer;
+			push(@output, "    </td>\n");
+			push(@output, "    </tr>\n");
+			push(@output, main::graph_footer());
 		}
-		print("  <br>\n");
-		return;
+		push(@output, "  <br>\n");
+		return @output;
 	}
 
 
@@ -431,42 +518,42 @@ sub system_cgi {
 		$u = "";
 	}
 
-	my $PNG1 = $u . $package . "1." . $tf->{when} . ".png";
-	my $PNG2 = $u . $package . "2." . $tf->{when} . ".png";
-	my $PNG3 = $u . $package . "3." . $tf->{when} . ".png";
-	my $PNG1z = $u . $package . "1z." . $tf->{when} . ".png";
-	my $PNG2z = $u . $package . "2z." . $tf->{when} . ".png";
-	my $PNG3z = $u . $package . "3z." . $tf->{when} . ".png";
-	unlink ("$PNG_DIR" . "$PNG1",
-		"$PNG_DIR" . "$PNG2",
-		"$PNG_DIR" . "$PNG3");
+	my $IMG1 = $u . $package . "1." . $tf->{when} . ".$imgfmt_lc";
+	my $IMG2 = $u . $package . "2." . $tf->{when} . ".$imgfmt_lc";
+	my $IMG3 = $u . $package . "3." . $tf->{when} . ".$imgfmt_lc";
+	my $IMG4 = $u . $package . "4." . $tf->{when} . ".$imgfmt_lc";
+	my $IMG5 = $u . $package . "5." . $tf->{when} . ".$imgfmt_lc";
+	my $IMG1z = $u . $package . "1z." . $tf->{when} . ".$imgfmt_lc";
+	my $IMG2z = $u . $package . "2z." . $tf->{when} . ".$imgfmt_lc";
+	my $IMG3z = $u . $package . "3z." . $tf->{when} . ".$imgfmt_lc";
+	my $IMG4z = $u . $package . "4z." . $tf->{when} . ".$imgfmt_lc";
+	my $IMG5z = $u . $package . "5z." . $tf->{when} . ".$imgfmt_lc";
+	unlink ("$IMG_DIR" . "$IMG1",
+		"$IMG_DIR" . "$IMG2",
+		"$IMG_DIR" . "$IMG3",
+		"$IMG_DIR" . "$IMG4",
+		"$IMG_DIR" . "$IMG5");
 	if(lc($config->{enable_zoom}) eq "y") {
-		unlink ("$PNG_DIR" . "$PNG1z",
-			"$PNG_DIR" . "$PNG2z",
-			"$PNG_DIR" . "$PNG3z");
+		unlink ("$IMG_DIR" . "$IMG1z",
+			"$IMG_DIR" . "$IMG2z",
+			"$IMG_DIR" . "$IMG3z",
+			"$IMG_DIR" . "$IMG4z",
+			"$IMG_DIR" . "$IMG5z");
 	}
 
 	if($title) {
-		main::graph_header($title, 2);
-	}
-	if(trim($rigid[0]) eq 1) {
-		push(@riglim, "--upper-limit=" . trim($limit[0]));
-	} else {
-		if(trim($rigid[0]) eq 2) {
-			push(@riglim, "--upper-limit=" . trim($limit[0]));
-			push(@riglim, "--rigid");
-		}
+		push(@output, main::graph_header($title, 2));
 	}
+	@riglim = @{setup_riglim($rigid[0], $limit[0])};
 	my $uptimeline;
 	if($RRDs::VERSION > 1.2) {
 		$uptimeline = "COMMENT:system uptime\\: " . get_uptime($config) . "\\c";
 	} else {
 		$uptimeline = "COMMENT:system uptime: " . get_uptime($config) . "\\c";
 	}
-
 	if($title) {
-		print("    <tr>\n");
-		print("    <td bgcolor='$colors->{title_bg_color}'>\n");
+		push(@output, "    <tr>\n");
+		push(@output, "    <td bgcolor='$colors->{title_bg_color}'>\n");
 	}
 	push(@tmp, "AREA:load1#4444EE: 1 min average");
 	push(@tmp, "GPRINT:load1:LAST:  Current\\: %4.2lf");
@@ -503,15 +590,14 @@ sub system_cgi {
 		@tmp = @tmpz;
 		push(@tmp, "COMMENT: \\n");
 	}
-	RRDs::graph("$PNG_DIR" . "$PNG1",
+	$pic = $rrd{$version}->("$IMG_DIR" . "$IMG1",
 		"--title=$config->{graphs}->{_system1}  ($tf->{nwhen}$tf->{twhen})",
 		"--start=-$tf->{nwhen}$tf->{twhen}",
-		"--imgformat=PNG",
+		"--imgformat=$imgfmt_uc",
 		"--vertical-label=Load average",
 		"--width=$width",
 		"--height=$height",
 		@riglim,
-		"--lower-limit=0",
 		$zoom,
 		@{$cgi->{version12}},
 		@{$colors->{graph_colors}},
@@ -524,18 +610,18 @@ sub system_cgi {
 		"COMMENT: \\n",
 		$uptimeline);
 	$err = RRDs::error;
-	print("ERROR: while graphing $PNG_DIR" . "$PNG1: $err\n") if $err;
+	push(@output, "ERROR: while graphing $IMG_DIR" . "$IMG1: $err\n") if $err;
 	if(lc($config->{enable_zoom}) eq "y") {
 		($width, $height) = split('x', $config->{graph_size}->{zoom});
-		RRDs::graph("$PNG_DIR" . "$PNG1z",
+		$picz = $rrd{$version}->("$IMG_DIR" . "$IMG1z",
 			"--title=$config->{graphs}->{_system1}  ($tf->{nwhen}$tf->{twhen})",
 			"--start=-$tf->{nwhen}$tf->{twhen}",
-			"--imgformat=PNG",
+			"--imgformat=$imgfmt_uc",
 			"--vertical-label=Load average",
 			"--width=$width",
 			"--height=$height",
 			@riglim,
-			"--lower-limit=0",
+			$zoom,
 			@{$cgi->{version12}},
 			@{$colors->{graph_colors}},
 			"DEF:load1=$rrd:system_load1:AVERAGE",
@@ -545,158 +631,68 @@ sub system_cgi {
 			@CDEF,
 			@tmpz);
 		$err = RRDs::error;
-		print("ERROR: while graphing $PNG_DIR" . "$PNG1z: $err\n") if $err;
+		push(@output, "ERROR: while graphing $IMG_DIR" . "$IMG1z: $err\n") if $err;
 	}
 	if($title || ($silent =~ /imagetag/ && $graph =~ /system1/)) {
 		if(lc($config->{enable_zoom}) eq "y") {
 			if(lc($config->{disable_javascript_void}) eq "y") {
-				print("      <a href=\"" . $config->{url} . "/" . $config->{imgs_dir} . $PNG1z . "\"><img src='" . $config->{url} . "/" . $config->{imgs_dir} . $PNG1 . "' border='0'></a>\n");
-			}
-			else {
-				print("      <a href=\"javascript:void(window.open('" . $config->{url} . "/" . $config->{imgs_dir} . $PNG1z . "','','width=" . ($width + 115) . ",height=" . ($height + 100) . ",scrollbars=0,resizable=0'))\"><img src='" . $config->{url} . "/" . $config->{imgs_dir} . $PNG1 . "' border='0'></a>\n");
+				push(@output, "      <a href=\"" . $config->{url} . "/" . $config->{imgs_dir} . $IMG1z . "\"><img src='" . $config->{url} . "/" . $config->{imgs_dir} . $IMG1 . "' border='0'></a>\n");
+			} else {
+				if($version eq "new") {
+					$picz_width = $picz->{image_width} * $config->{global_zoom};
+					$picz_height = $picz->{image_height} * $config->{global_zoom};
+				} else {
+					$picz_width = $width + 115;
+					$picz_height = $height + 100;
+				}
+				push(@output, "      <a href=\"javascript:void(window.open('" . $config->{url} . "/" . $config->{imgs_dir} . $IMG1z . "','','width=" . $picz_width . ",height=" . $picz_height . ",scrollbars=0,resizable=0'))\"><img src='" . $config->{url} . "/" . $config->{imgs_dir} . $IMG1 . "' border='0'></a>\n");
 			}
 		} else {
-			print("      <img src='" . $config->{url} . "/" . $config->{imgs_dir} . $PNG1 . "'>\n");
-		}
-	}
-
-	if($title) {
-		print("    </td>\n");
-		print("    <td bgcolor='" . $colors->{title_bg_color} . "'>\n");
-	}
-	undef(@riglim);
-	if(trim($rigid[1]) eq 1) {
-		push(@riglim, "--upper-limit=" . trim($limit[1]));
-	} else {
-		if(trim($rigid[1]) eq 2) {
-			push(@riglim, "--upper-limit=" . trim($limit[1]));
-			push(@riglim, "--rigid");
-		}
-	}
-	undef(@tmp);
-	undef(@tmpz);
-	undef(@CDEF);
-	push(@tmp, "AREA:npslp#44AAEE:Sleeping");
-	push(@tmp, "AREA:nprun#EE4444:Running");
-	push(@tmp, "LINE1:nprun#EE0000");
-	push(@tmp, "LINE1:npslp#00EEEE");
-	push(@tmp, "LINE1:nproc#EEEE00:Processes");
-	if(lc($config->{show_gaps}) eq "y") {
-		push(@tmp, "AREA:wrongdata#$colors->{gap}:");
-		push(@CDEF, "CDEF:wrongdata=allvalues,UN,INF,UNKN,IF");
-	}
-	($width, $height) = split('x', $config->{graph_size}->{small});
-	if($silent =~ /imagetag/) {
-		($width, $height) = split('x', $config->{graph_size}->{remote}) if $silent eq "imagetag";
-		($width, $height) = split('x', $config->{graph_size}->{main}) if $silent eq "imagetagbig";
-		push(@tmp, "COMMENT: \\n");
-		push(@tmp, "COMMENT: \\n");
-	}
-	RRDs::graph("$PNG_DIR" . "$PNG2",
-		"--title=$config->{graphs}->{_system2}  ($tf->{nwhen}$tf->{twhen})",
-		"--start=-$tf->{nwhen}$tf->{twhen}",
-		"--imgformat=PNG",
-		"--vertical-label=Processes",
-		"--width=$width",
-		"--height=$height",
-		@riglim,
-		"--lower-limit=0",
-		$zoom,
-		@{$cgi->{version12}},
-		@{$cgi->{version12_small}},
-		@{$colors->{graph_colors}},
-		"DEF:nproc=$rrd:system_nproc:AVERAGE",
-		"DEF:npslp=$rrd:system_npslp:AVERAGE",
-		"DEF:nprun=$rrd:system_nprun:AVERAGE",
-		"CDEF:allvalues=nproc,npslp,nprun,+,+",
-		@CDEF,
-		@tmp,
-		"COMMENT: \\n");
-	$err = RRDs::error;
-	print("ERROR: while graphing $PNG_DIR" . "$PNG2: $err\n") if $err;
-	if(lc($config->{enable_zoom}) eq "y") {
-		($width, $height) = split('x', $config->{graph_size}->{zoom});
-		RRDs::graph("$PNG_DIR" . "$PNG2z",
-			"--title=$config->{graphs}->{_system2}  ($tf->{nwhen}$tf->{twhen})",
-			"--start=-$tf->{nwhen}$tf->{twhen}",
-			"--imgformat=PNG",
-			"--vertical-label=Processes",
-			"--width=$width",
-			"--height=$height",
-			@riglim,
-			"--lower-limit=0",
-			@{$cgi->{version12}},
-			@{$cgi->{version12_small}},
-			@{$colors->{graph_colors}},
-			"DEF:nproc=$rrd:system_nproc:AVERAGE",
-			"DEF:npslp=$rrd:system_npslp:AVERAGE",
-			"DEF:nprun=$rrd:system_nprun:AVERAGE",
-			"CDEF:allvalues=nproc,npslp,nprun,+,+",
-			@CDEF,
-			@tmp);
-		$err = RRDs::error;
-		print("ERROR: while graphing $PNG_DIR" . "$PNG2z: $err\n") if $err;
-	}
-	if($title || ($silent =~ /imagetag/ && $graph =~ /system2/)) {
-		if(lc($config->{enable_zoom}) eq "y") {
-			if(lc($config->{disable_javascript_void}) eq "y") {
-				print("      <a href=\"" . $config->{url} . "/" . $config->{imgs_dir} . $PNG2z . "\"><img src='" . $config->{url} . "/" . $config->{imgs_dir} . $PNG2 . "' border='0'></a>\n");
-			}
-			else {
-				print("      <a href=\"javascript:void(window.open('" . $config->{url} . "/" . $config->{imgs_dir} . $PNG2z . "','','width=" . ($width + 115) . ",height=" . ($height + 100) . ",scrollbars=0,resizable=0'))\"><img src='" . $config->{url} . "/" . $config->{imgs_dir} . $PNG2 . "' border='0'></a>\n");
-			}
-		} else {
-			print("      <img src='" . $config->{url} . "/" . $config->{imgs_dir} . $PNG2 . "'>\n");
+			push(@output, "      <img src='" . $config->{url} . "/" . $config->{imgs_dir} . $IMG1 . "'>\n");
 		}
 	}
 
 	undef(@tmp);
 	undef(@tmpz);
 	undef(@CDEF);
-	if($config->{os} eq "Linux") {
+	if($config->{os} eq "Linux" || $config->{os} eq "FreeBSD") {
 		push(@tmp, "AREA:m_mused#EE4444:Used");
 		push(@tmp, "AREA:m_mcach#44EE44:Cached");
 		push(@tmp, "AREA:m_mbuff#CCCCCC:Buffers");
-		push(@tmp, "LINE1:m_mbuff#888888");
-		push(@tmp, "LINE1:m_mcach#00EE00");
-		push(@tmp, "LINE1:m_mused#EE0000");
-	} elsif($config->{os} eq "FreeBSD") {
-		push(@tmp, "AREA:m_mused#EE4444:Used");
-		push(@tmp, "AREA:m_mcach#44EE44:Cached");
-		push(@tmp, "AREA:m_mbuff#CCCCCC:Buffers");
-		push(@tmp, "AREA:m_macti#EEEE44:Active");
-		push(@tmp, "AREA:m_minac#4444EE:Inactive");
-		push(@tmp, "LINE1:m_minac#0000EE");
-		push(@tmp, "LINE1:m_macti#EEEE00");
-		push(@tmp, "LINE1:m_mbuff#888888");
-		push(@tmp, "LINE1:m_mcach#00EE00");
-		push(@tmp, "LINE1:m_mused#EE0000");
+		push(@tmp, "AREA:m_macti#E29136:Active");
+		push(@tmp, "AREA:m_minac#448844:Inactive");
+		push(@tmp, "LINE2:m_minac#008800");
+		push(@tmp, "LINE2:m_macti#E29136");
+		push(@tmp, "LINE2:m_mbuff#888888");
+		push(@tmp, "LINE2:m_mcach#00EE00");
+		push(@tmp, "LINE2:m_mused#EE0000");
 	} elsif($config->{os} eq "OpenBSD" || $config->{os} eq "NetBSD") {
 		push(@tmp, "AREA:m_mused#EE4444:Used");
 		push(@tmp, "AREA:m_macti#44EE44:Active");
-		push(@tmp, "LINE1:m_macti#00EE00");
-		push(@tmp, "LINE1:m_mused#EE0000");
+		push(@tmp, "LINE2:m_macti#00EE00");
+		push(@tmp, "LINE2:m_mused#EE0000");
 	}
 	if(lc($config->{show_gaps}) eq "y") {
 		push(@tmp, "AREA:wrongdata#$colors->{gap}:");
 		push(@CDEF, "CDEF:wrongdata=allvalues,UN,INF,UNKN,IF");
 	}
-	($width, $height) = split('x', $config->{graph_size}->{small});
+	($width, $height) = split('x', $config->{graph_size}->{main});
 	if($silent =~ /imagetag/) {
 		($width, $height) = split('x', $config->{graph_size}->{remote}) if $silent eq "imagetag";
 		($width, $height) = split('x', $config->{graph_size}->{main}) if $silent eq "imagetagbig";
 		push(@tmp, "COMMENT: \\n");
 		push(@tmp, "COMMENT: \\n");
 	}
-	RRDs::graph("$PNG_DIR" . "$PNG3",
-		"--title=$config->{graphs}->{_system3} (${total_mem}MB)  ($tf->{nwhen}$tf->{twhen})",
+	$pic = $rrd{$version}->("$IMG_DIR" . "$IMG2",
+		"--title=$config->{graphs}->{_system2} (${total_mem}MB)  ($tf->{nwhen}$tf->{twhen})",
 		"--start=-$tf->{nwhen}$tf->{twhen}",
-		"--imgformat=PNG",
-		"--vertical-label=Megabytes",
+		"--imgformat=$imgfmt_uc",
+		"--vertical-label=bytes",
 		"--width=$width",
 		"--height=$height",
-		"--upper-limit=$total_mem",
+		"--upper-limit=$total_mem_bytes",
 		"--lower-limit=0",
+		"--rigid",
 		"--base=1024",
 		$zoom,
 		@{$cgi->{version12}},
@@ -708,30 +704,32 @@ sub system_cgi {
 		"DEF:mfree=$rrd:system_mfree:AVERAGE",
 		"DEF:macti=$rrd:system_macti:AVERAGE",
 		"DEF:minac=$rrd:system_minac:AVERAGE",
-		"CDEF:m_mtotl=mtotl,1024,/",
-		"CDEF:m_mbuff=mbuff,1024,/",
-		"CDEF:m_mcach=mcach,1024,/",
-		"CDEF:m_mused=m_mtotl,mfree,1024,/,-",
-		"CDEF:m_macti=macti,1024,/",
-		"CDEF:m_minac=minac,1024,/",
+		"CDEF:m_mtotl=mtotl,1024,*",
+		"CDEF:m_mbuff=mbuff,1024,*",
+		"CDEF:m_mcach=mcach,1024,*",
+		"CDEF:m_mused=m_mtotl,mfree,1024,*,-",
+		"CDEF:m_macti=macti,1024,*",
+		"CDEF:m_minac=minac,1024,*",
 		"CDEF:allvalues=mtotl,mbuff,mcach,mfree,macti,minac,+,+,+,+,+",
 		@CDEF,
 		@tmp,
 		"COMMENT: \\n");
 	$err = RRDs::error;
-	print("ERROR: while graphing $PNG_DIR" . "$PNG3: $err\n") if $err;
+	push(@output, "ERROR: while graphing $IMG_DIR" . "$IMG2: $err\n") if $err;
 	if(lc($config->{enable_zoom}) eq "y") {
 		($width, $height) = split('x', $config->{graph_size}->{zoom});
-		RRDs::graph("$PNG_DIR" . "$PNG3z",
-			"--title=$config->{graphs}->{_system3} (${total_mem}MB)  ($tf->{nwhen}$tf->{twhen})",
+		$picz = $rrd{$version}->("$IMG_DIR" . "$IMG2z",
+			"--title=$config->{graphs}->{_system2} (${total_mem}MB)  ($tf->{nwhen}$tf->{twhen})",
 			"--start=-$tf->{nwhen}$tf->{twhen}",
-			"--imgformat=PNG",
-			"--vertical-label=Megabytes",
+			"--imgformat=$imgfmt_uc",
+			"--vertical-label=bytes",
 			"--width=$width",
 			"--height=$height",
-			"--upper-limit=$total_mem",
+			"--upper-limit=$total_mem_bytes",
 			"--lower-limit=0",
+			"--rigid",
 			"--base=1024",
+			$zoom,
 			@{$cgi->{version12}},
 			@{$cgi->{version12_small}},
 			@{$colors->{graph_colors}},
@@ -741,38 +739,326 @@ sub system_cgi {
 			"DEF:mfree=$rrd:system_mfree:AVERAGE",
 			"DEF:macti=$rrd:system_macti:AVERAGE",
 			"DEF:minac=$rrd:system_minac:AVERAGE",
-			"CDEF:m_mtotl=mtotl,1024,/",
-			"CDEF:m_mbuff=mbuff,1024,/",
-			"CDEF:m_mcach=mcach,1024,/",
-			"CDEF:m_mused=m_mtotl,mfree,1024,/,-",
-			"CDEF:m_macti=macti,1024,/",
-			"CDEF:m_minac=minac,1024,/",
+			"CDEF:m_mtotl=mtotl,1024,*",
+			"CDEF:m_mbuff=mbuff,1024,*",
+			"CDEF:m_mcach=mcach,1024,*",
+			"CDEF:m_mused=m_mtotl,mfree,1024,*,-",
+			"CDEF:m_macti=macti,1024,*",
+			"CDEF:m_minac=minac,1024,*",
 			"CDEF:allvalues=mtotl,mbuff,mcach,mfree,macti,minac,+,+,+,+,+",
 			@CDEF,
 			@tmp);
 		$err = RRDs::error;
-		print("ERROR: while graphing $PNG_DIR" . "$PNG3z: $err\n") if $err;
+		push(@output, "ERROR: while graphing $IMG_DIR" . "$IMG2z: $err\n") if $err;
 	}
-	if($title || ($silent =~ /imagetag/ && $graph =~ /system3/)) {
+	if($title || ($silent =~ /imagetag/ && $graph =~ /system2/)) {
 		if(lc($config->{enable_zoom}) eq "y") {
 			if(lc($config->{disable_javascript_void}) eq "y") {
-				print("      <a href=\"" . $config->{url} . "/" . $config->{imgs_dir} . $PNG3z . "\"><img src='" . $config->{url} . "/" . $config->{imgs_dir} . $PNG3 . "' border='0'></a>\n");
-			}
-			else {
-				print("      <a href=\"javascript:void(window.open('" . $config->{url} . "/" . $config->{imgs_dir} . $PNG3z . "','','width=" . ($width + 115) . ",height=" . ($height + 100) . ",scrollbars=0,resizable=0'))\"><img src='" . $config->{url} . "/" . $config->{imgs_dir} . $PNG3 . "' border='0'></a>\n");
+				push(@output, "      <a href=\"" . $config->{url} . "/" . $config->{imgs_dir} . $IMG2z . "\"><img src='" . $config->{url} . "/" . $config->{imgs_dir} . $IMG2 . "' border='0'></a>\n");
+			} else {
+				if($version eq "new") {
+					$picz_width = $picz->{image_width} * $config->{global_zoom};
+					$picz_height = $picz->{image_height} * $config->{global_zoom};
+				} else {
+					$picz_width = $width + 115;
+					$picz_height = $height + 100;
+				}
+				push(@output, "      <a href=\"javascript:void(window.open('" . $config->{url} . "/" . $config->{imgs_dir} . $IMG2z . "','','width=" . $picz_width . ",height=" . $picz_height . ",scrollbars=0,resizable=0'))\"><img src='" . $config->{url} . "/" . $config->{imgs_dir} . $IMG2 . "' border='0'></a>\n");
 			}
 		} else {
-			print("      <img src='" . $config->{url} . "/" . $config->{imgs_dir} . $PNG3 . "'>\n");
+			push(@output, "      <img src='" . $config->{url} . "/" . $config->{imgs_dir} . $IMG2 . "'>\n");
 		}
 	}
 
 	if($title) {
-		print("    </td>\n");
-		print("    </tr>\n");
-		main::graph_footer();
+		push(@output, "    </td>\n");
+		push(@output, "    <td valign='top' bgcolor='" . $colors->{title_bg_color} . "'>\n");
 	}
-	print("  <br>\n");
-	return;
+
+	@riglim = @{setup_riglim($rigid[2], $limit[2])};
+	undef(@tmp);
+	undef(@tmpz);
+	undef(@CDEF);
+	if($config->{os} eq "Linux") {
+		push(@tmp, "AREA:npslp#448844:Sleeping");
+		push(@tmp, "GPRINT:npslp:LAST:             Current\\:%5.0lf\\n");
+		push(@tmp, "LINE2:npwio#EE44EE:Wait I/O");
+		push(@tmp, "GPRINT:npwio:LAST:             Current\\:%5.0lf\\n");
+		push(@tmp, "LINE2:npzom#00EEEE:Zombie");
+		push(@tmp, "GPRINT:npzom:LAST:               Current\\:%5.0lf\\n");
+		push(@tmp, "LINE2:npstp#EEEE00:Stopped");
+		push(@tmp, "GPRINT:npstp:LAST:              Current\\:%5.0lf\\n");
+		push(@tmp, "LINE2:npswp#0000EE:Paging");
+		push(@tmp, "GPRINT:npswp:LAST:               Current\\:%5.0lf\\n");
+		push(@tmp, "LINE2:nprun#EE0000:Running");
+		push(@tmp, "GPRINT:nprun:LAST:              Current\\:%5.0lf\\n");
+		push(@tmp, "COMMENT: \\n");
+		push(@tmp, "LINE2:nproc#888888:Total Processes");
+		push(@tmp, "GPRINT:nproc:LAST:      Current\\:%5.0lf\\n");
+		push(@tmpz, "AREA:npslp#448844:Sleeping");
+		push(@tmpz, "LINE2:npwio#EE44EE:Wait I/O");
+		push(@tmpz, "LINE2:npzom#00EEEE:Zombie");
+		push(@tmpz, "LINE2:npstp#EEEE00:Stopped");
+		push(@tmpz, "LINE2:npswp#0000EE:Paging");
+		push(@tmpz, "LINE2:nprun#EE0000:Running");
+		push(@tmpz, "LINE2:nproc#888888:Total Processes");
+	} else {
+		push(@tmp, "AREA:npslp#44AAEE:Sleeping");
+		push(@tmp, "AREA:nprun#EE4444:Running");
+		push(@tmp, "LINE1:nprun#EE0000");
+		push(@tmp, "LINE1:npslp#00EEEE");
+		push(@tmp, "LINE1:nproc#EEEE00:Processes");
+		push(@tmpz, "AREA:npslp#44AAEE:Sleeping");
+		push(@tmpz, "AREA:nprun#EE4444:Running");
+		push(@tmpz, "LINE1:nprun#EE0000");
+		push(@tmpz, "LINE1:npslp#00EEEE");
+		push(@tmpz, "LINE1:nproc#EEEE00:Processes");
+	}
+	if(lc($config->{show_gaps}) eq "y") {
+		push(@tmp, "AREA:wrongdata#$colors->{gap}:");
+		push(@tmpz, "AREA:wrongdata#$colors->{gap}:");
+		push(@CDEF, "CDEF:wrongdata=allvalues,UN,INF,UNKN,IF");
+	}
+	($width, $height) = split('x', $config->{graph_size}->{small});
+	if($silent =~ /imagetag/) {
+		($width, $height) = split('x', $config->{graph_size}->{remote}) if $silent eq "imagetag";
+		($width, $height) = split('x', $config->{graph_size}->{main}) if $silent eq "imagetagbig";
+		push(@tmp, "COMMENT: \\n");
+		push(@tmp, "COMMENT: \\n");
+	}
+	$pic = $rrd{$version}->("$IMG_DIR" . "$IMG3",
+		"--title=$config->{graphs}->{_system3}  ($tf->{nwhen}$tf->{twhen})",
+		"--start=-$tf->{nwhen}$tf->{twhen}",
+		"--imgformat=$imgfmt_uc",
+		"--vertical-label=Processes",
+		"--width=$width",
+		"--height=$height",
+		@riglim,
+		$zoom,
+		@{$cgi->{version12}},
+		@{$cgi->{version12_small}},
+		@{$colors->{graph_colors}},
+		"DEF:nproc=$rrd:system_nproc:AVERAGE",
+		"DEF:npslp=$rrd:system_npslp:AVERAGE",
+		"DEF:nprun=$rrd:system_nprun:AVERAGE",
+		"DEF:npwio=$rrd:system_npwio:AVERAGE",
+		"DEF:npzom=$rrd:system_npzom:AVERAGE",
+		"DEF:npstp=$rrd:system_npstp:AVERAGE",
+		"DEF:npswp=$rrd:system_npswp:AVERAGE",
+		"CDEF:allvalues=nproc,npslp,nprun,npwio,npzom,npstp,npswp,+,+,+,+,+,+",
+		@CDEF,
+		@tmp);
+	$err = RRDs::error;
+	push(@output, "ERROR: while graphing $IMG_DIR" . "$IMG3: $err\n") if $err;
+	if(lc($config->{enable_zoom}) eq "y") {
+		($width, $height) = split('x', $config->{graph_size}->{zoom});
+		$picz = $rrd{$version}->("$IMG_DIR" . "$IMG3z",
+			"--title=$config->{graphs}->{_system3}  ($tf->{nwhen}$tf->{twhen})",
+			"--start=-$tf->{nwhen}$tf->{twhen}",
+			"--imgformat=$imgfmt_uc",
+			"--vertical-label=Processes",
+			"--width=$width",
+			"--height=$height",
+			@riglim,
+			$zoom,
+			@{$cgi->{version12}},
+			@{$cgi->{version12_small}},
+			@{$colors->{graph_colors}},
+			"DEF:nproc=$rrd:system_nproc:AVERAGE",
+			"DEF:npslp=$rrd:system_npslp:AVERAGE",
+			"DEF:nprun=$rrd:system_nprun:AVERAGE",
+			"DEF:npwio=$rrd:system_npwio:AVERAGE",
+			"DEF:npzom=$rrd:system_npzom:AVERAGE",
+			"DEF:npstp=$rrd:system_npstp:AVERAGE",
+			"DEF:npswp=$rrd:system_npswp:AVERAGE",
+			"CDEF:allvalues=nproc,npslp,nprun,npwio,npzom,npstp,npswp,+,+,+,+,+,+",
+			@CDEF,
+			@tmpz);
+		$err = RRDs::error;
+		push(@output, "ERROR: while graphing $IMG_DIR" . "$IMG3z: $err\n") if $err;
+	}
+	if($title || ($silent =~ /imagetag/ && $graph =~ /system3/)) {
+		if(lc($config->{enable_zoom}) eq "y") {
+			if(lc($config->{disable_javascript_void}) eq "y") {
+				push(@output, "      <a href=\"" . $config->{url} . "/" . $config->{imgs_dir} . $IMG3z . "\"><img src='" . $config->{url} . "/" . $config->{imgs_dir} . $IMG3 . "' border='0'></a>\n");
+			} else {
+				if($version eq "new") {
+					$picz_width = $picz->{image_width} * $config->{global_zoom};
+					$picz_height = $picz->{image_height} * $config->{global_zoom};
+				} else {
+					$picz_width = $width + 115;
+					$picz_height = $height + 100;
+				}
+				push(@output, "      <a href=\"javascript:void(window.open('" . $config->{url} . "/" . $config->{imgs_dir} . $IMG3z . "','','width=" . $picz_width . ",height=" . $picz_height . ",scrollbars=0,resizable=0'))\"><img src='" . $config->{url} . "/" . $config->{imgs_dir} . $IMG3 . "' border='0'></a>\n");
+			}
+		} else {
+			push(@output, "      <img src='" . $config->{url} . "/" . $config->{imgs_dir} . $IMG3 . "'>\n");
+		}
+	}
+
+	@riglim = @{setup_riglim($rigid[3], $limit[3])};
+	undef(@tmp);
+	undef(@tmpz);
+	undef(@CDEF);
+	push(@tmp, "LINE2:entropy#EEEE00:Entropy");
+	push(@tmp, "GPRINT:entropy:LAST:              Current\\:%5.0lf\\n");
+	push(@tmpz, "LINE2:entropy#EEEE00:Entropy");
+	if(lc($config->{show_gaps}) eq "y") {
+		push(@tmp, "AREA:wrongdata#$colors->{gap}:");
+		push(@tmpz, "AREA:wrongdata#$colors->{gap}:");
+		push(@CDEF, "CDEF:wrongdata=allvalues,UN,INF,UNKN,IF");
+	}
+	($width, $height) = split('x', $config->{graph_size}->{small});
+	if($silent =~ /imagetag/) {
+		($width, $height) = split('x', $config->{graph_size}->{remote}) if $silent eq "imagetag";
+		($width, $height) = split('x', $config->{graph_size}->{main}) if $silent eq "imagetagbig";
+		push(@tmp, "COMMENT: \\n");
+		push(@tmp, "COMMENT: \\n");
+	}
+	$pic = $rrd{$version}->("$IMG_DIR" . "$IMG4",
+		"--title=$config->{graphs}->{_system4}  ($tf->{nwhen}$tf->{twhen})",
+		"--start=-$tf->{nwhen}$tf->{twhen}",
+		"--imgformat=$imgfmt_uc",
+		"--vertical-label=Size",
+		"--width=$width",
+		"--height=$height",
+		@riglim,
+		$zoom,
+		@{$cgi->{version12}},
+		@{$cgi->{version12_small}},
+		@{$colors->{graph_colors}},
+		"DEF:entropy=$rrd:system_entrop:AVERAGE",
+		"CDEF:allvalues=entropy",
+		@CDEF,
+		@tmp);
+	$err = RRDs::error;
+	push(@output, "ERROR: while graphing $IMG_DIR" . "$IMG4: $err\n") if $err;
+	if(lc($config->{enable_zoom}) eq "y") {
+		($width, $height) = split('x', $config->{graph_size}->{zoom});
+		$picz = $rrd{$version}->("$IMG_DIR" . "$IMG4z",
+			"--title=$config->{graphs}->{_system4}  ($tf->{nwhen}$tf->{twhen})",
+			"--start=-$tf->{nwhen}$tf->{twhen}",
+			"--imgformat=$imgfmt_uc",
+			"--vertical-label=Size",
+			"--width=$width",
+			"--height=$height",
+			@riglim,
+			$zoom,
+			@{$cgi->{version12}},
+			@{$cgi->{version12_small}},
+			@{$colors->{graph_colors}},
+			"DEF:entropy=$rrd:system_entrop:AVERAGE",
+			"CDEF:allvalues=entropy",
+			@CDEF,
+			@tmpz);
+		$err = RRDs::error;
+		push(@output, "ERROR: while graphing $IMG_DIR" . "$IMG4z: $err\n") if $err;
+	}
+	if($title || ($silent =~ /imagetag/ && $graph =~ /system4/)) {
+		if(lc($config->{enable_zoom}) eq "y") {
+			if(lc($config->{disable_javascript_void}) eq "y") {
+				push(@output, "      <a href=\"" . $config->{url} . "/" . $config->{imgs_dir} . $IMG4z . "\"><img src='" . $config->{url} . "/" . $config->{imgs_dir} . $IMG4 . "' border='0'></a>\n");
+			} else {
+				if($version eq "new") {
+					$picz_width = $picz->{image_width} * $config->{global_zoom};
+					$picz_height = $picz->{image_height} * $config->{global_zoom};
+				} else {
+					$picz_width = $width + 115;
+					$picz_height = $height + 100;
+				}
+				push(@output, "      <a href=\"javascript:void(window.open('" . $config->{url} . "/" . $config->{imgs_dir} . $IMG4z . "','','width=" . $picz_width . ",height=" . $picz_height . ",scrollbars=0,resizable=0'))\"><img src='" . $config->{url} . "/" . $config->{imgs_dir} . $IMG4 . "' border='0'></a>\n");
+			}
+		} else {
+			push(@output, "      <img src='" . $config->{url} . "/" . $config->{imgs_dir} . $IMG4 . "'>\n");
+		}
+	}
+
+	@riglim = @{setup_riglim($rigid[4], $limit[4])};
+	undef(@tmp);
+	undef(@tmpz);
+	undef(@CDEF);
+	push(@tmp, "LINE2:uptime_days#EE44EE:Uptime");
+	push(@tmp, "GPRINT:uptime_days:LAST:(in days)      Current\\:%5.1lf\\n");
+	push(@tmpz, "LINE2:uptime_days#EE44EE:Uptime");
+	if(lc($config->{show_gaps}) eq "y") {
+		push(@tmp, "AREA:wrongdata#$colors->{gap}:");
+		push(@tmpz, "AREA:wrongdata#$colors->{gap}:");
+		push(@CDEF, "CDEF:wrongdata=allvalues,UN,INF,UNKN,IF");
+	}
+	($width, $height) = split('x', $config->{graph_size}->{small});
+	if($silent =~ /imagetag/) {
+		($width, $height) = split('x', $config->{graph_size}->{remote}) if $silent eq "imagetag";
+		($width, $height) = split('x', $config->{graph_size}->{main}) if $silent eq "imagetagbig";
+		push(@tmp, "COMMENT: \\n");
+		push(@tmp, "COMMENT: \\n");
+	}
+	$pic = $rrd{$version}->("$IMG_DIR" . "$IMG5",
+		"--title=$config->{graphs}->{_system5}  ($tf->{nwhen}$tf->{twhen})",
+		"--start=-$tf->{nwhen}$tf->{twhen}",
+		"--imgformat=$imgfmt_uc",
+		"--vertical-label=Days",
+		"--width=$width",
+		"--height=$height",
+		@riglim,
+		$zoom,
+		@{$cgi->{version12}},
+		@{$cgi->{version12_small}},
+		@{$colors->{graph_colors}},
+		"DEF:uptime=$rrd:system_uptime:AVERAGE",
+		"CDEF:uptime_days=uptime,86400,/",
+		"CDEF:allvalues=uptime",
+		@CDEF,
+		@tmp);
+	$err = RRDs::error;
+	push(@output, "ERROR: while graphing $IMG_DIR" . "$IMG5: $err\n") if $err;
+	if(lc($config->{enable_zoom}) eq "y") {
+		($width, $height) = split('x', $config->{graph_size}->{zoom});
+		$picz = $rrd{$version}->("$IMG_DIR" . "$IMG5z",
+			"--title=$config->{graphs}->{_system5}  ($tf->{nwhen}$tf->{twhen})",
+			"--start=-$tf->{nwhen}$tf->{twhen}",
+			"--imgformat=$imgfmt_uc",
+			"--vertical-label=Days",
+			"--width=$width",
+			"--height=$height",
+			@riglim,
+			$zoom,
+			@{$cgi->{version12}},
+			@{$cgi->{version12_small}},
+			@{$colors->{graph_colors}},
+			"DEF:uptime=$rrd:system_uptime:AVERAGE",
+			"CDEF:uptime_days=uptime,86400,/",
+			"CDEF:allvalues=uptime",
+			@CDEF,
+			@tmpz);
+		$err = RRDs::error;
+		push(@output, "ERROR: while graphing $IMG_DIR" . "$IMG5z: $err\n") if $err;
+	}
+	if($title || ($silent =~ /imagetag/ && $graph =~ /system5/)) {
+		if(lc($config->{enable_zoom}) eq "y") {
+			if(lc($config->{disable_javascript_void}) eq "y") {
+				push(@output, "      <a href=\"" . $config->{url} . "/" . $config->{imgs_dir} . $IMG5z . "\"><img src='" . $config->{url} . "/" . $config->{imgs_dir} . $IMG5 . "' border='0'></a>\n");
+			} else {
+				if($version eq "new") {
+					$picz_width = $picz->{image_width} * $config->{global_zoom};
+					$picz_height = $picz->{image_height} * $config->{global_zoom};
+				} else {
+					$picz_width = $width + 115;
+					$picz_height = $height + 100;
+				}
+				push(@output, "      <a href=\"javascript:void(window.open('" . $config->{url} . "/" . $config->{imgs_dir} . $IMG5z . "','','width=" . $picz_width . ",height=" . $picz_height . ",scrollbars=0,resizable=0'))\"><img src='" . $config->{url} . "/" . $config->{imgs_dir} . $IMG5 . "' border='0'></a>\n");
+			}
+		} else {
+			push(@output, "      <img src='" . $config->{url} . "/" . $config->{imgs_dir} . $IMG5 . "'>\n");
+		}
+	}
+
+	if($title) {
+		push(@output, "    </td>\n");
+		push(@output, "    </tr>\n");
+		push(@output, main::graph_footer());
+	}
+	push(@output, "  <br>\n");
+	return @output;
 }
 
 sub get_uptime {
